/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005-2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.repository.web.client.internal.RepositoryClient");

dojo.require("com.ibm.team.repository.web.transport.ServiceRequest");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.transport.TeamServerClient");

(function() {

/**
 * Provides helper methods to fetch data from the repository package.
 */
com.ibm.team.repository.web.client.internal.RepositoryClient= {
	
	SERVICE_URI: "com.ibm.team.repository.common.internal.IContributorRestService",
	INIT_SERVICE_URI: "com.ibm.team.repository.service.internal.webuiInitializer.IWebUIInitializerRestService",
	
	GET_CONTRIBUTORS_METHOD: "getContributors",
	POST_CONTRIBUTOR_METHOD: "postContributor",
	QUERY_CONTRIBUTORS_URL: "/resource/virtual/contributors",

    /*
     * (non-Javadoc)
     * @see com.ibm.team.repository.common.internal.IContributorRestService#getContributors(com.ibm.team.repository.common.internal.IContributorRestService.ParmsGetContributors)
     */
	getContributors: function(responseHandler, appArgs) {
		this._invokeRepositoryService(responseHandler, this.GET_CONTRIBUTORS_METHOD, appArgs);
	},
	
    /*
     * This is a helper method to access a crud service used to query contributors by a jsonExpression.
     */
	getContributorQueryResults: function(responseHandler, appArgs) {
		var url = [];
		url.push(net.jazz.ajax._contextRoot);
		url.push(this.QUERY_CONTRIBUTORS_URL);
		if (appArgs.expression)
			appArgs.expression = dojo.toJson(appArgs.expression);
		var first = true;
		for (var i in appArgs) {
			if (first)
				url.push("?");
			else
				url.push("&");
			first = false;
			url.push(i);
			url.push("=");
			url.push(encodeURIComponent(appArgs[i]));
		}
		var bindArgs = {
			url: url.join(""),
			handleAs: "json",
			headers: {accept: "text/json"},
			handle: function(response, ioArgs){
				if (response instanceof Error) {
					responseHandler.doErrorCallback({
						status: ioArgs.xhr.status,
						message: ioArgs.xhr.statusText
					});
				}
				else {
					responseHandler.doCallback(response);
				}
			}
		};
		dojo.xhrGet(bindArgs);
	},
	
	/*
     * (non-Javadoc)
     * @see com.ibm.team.repository.common.internal.IContributorRestService#postContributor(com.ibm.team.repository.common.internal.IContributorRestService.ParmsPostContributor)
     */
	postContributor: function(responseHandler, appArgs) {
		if (!appArgs.name) {
			alert("Missing required field 'name'");
			return;
		}
		if (!appArgs.userId) {
			alert("Missing required field 'userId'");
			return;
		}
		this._invokeRepositoryService(responseHandler, this.POST_CONTRIBUTOR_METHOD, appArgs);
	},
	
	getAuthenticatedContributor: function(responseHandler) {
		this._invokeRepositoryService(responseHandler, "getCurrentContributor");
	},
	
	getInitializationData: function(responseHandler) {
		var serviceRequest = new com.ibm.team.repository.web.transport.ServiceRequest(this.INIT_SERVICE_URI, "getInitializationData");
		com.ibm.team.repository.web.transport.TeamServerClient.invokeService(serviceRequest, responseHandler);
	},
		
	_invokeRepositoryService: function(responseHandler, method, appArgs) {
		// appArgs is optional, so test whether or not it was sent
		if (arguments.length == 2) {
			var serviceRequest= new com.ibm.team.repository.web.transport.ServiceRequest(
					this.SERVICE_URI,
					method);
		} else {
			serviceRequest= new com.ibm.team.repository.web.transport.ServiceRequest(
					this.SERVICE_URI,
					method,
					appArgs);
		}
		com.ibm.team.repository.web.transport.TeamServerClient.invokeService(
				serviceRequest,
				responseHandler);
	}
	
//	postDeleteContributor: function(responseHandler, appArgs) {
//		if (!appArgs.itemId) {
//			dojo.raise("RepositoryClient#postDeleteContributor: missing parameter 'itemId'");
//		}
//		this._invokeRepositoryService(responseHandler, "postDeleteContributor", appArgs);
//	}
};

})();
