/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005-2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.team.repository.web.ui.internal.ProjectSelector");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.i18n");
dojo.require("dojo.string");

dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.transport.ServiceRequest");
dojo.require("com.ibm.team.repository.web.transport.TeamServerClient");
dojo.require("com.ibm.team.repository.web.ui.internal.Util");
dojo.require("net.jazz.ajax.ui.PlatformUI");

dojo.requireLocalization("com.ibm.team.repository.web.ui.internal", "ProjectSelectorMessages");

var getAuthenticatedUserId 		= com.ibm.team.repository.web.client.session.getAuthenticatedUserId;
var ServiceRequest				= com.ibm.team.repository.web.transport.ServiceRequest;
var ServiceResponseHandler		= com.ibm.team.repository.web.transport.ServiceResponseHandler;
var TeamServerClient 			= com.ibm.team.repository.web.transport.TeamServerClient;
var Util						= com.ibm.team.repository.web.ui.internal.Util;
var SERVICE_URI      			= "com.ibm.team.process.internal.common.service.IProcessRestService";
var PlatformUI 					= net.jazz.ajax.ui.PlatformUI;

dojo.declare("com.ibm.team.repository.web.ui.internal.ProjectSelector", [dijit._Widget, dijit._Templated], {
	
	//
	// Module-private properties and methods
	//

	templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/templates/ProjectSelector.html"),
	
	constructor: function () {
		this.messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal", "ProjectSelectorMessages");
	},
		
	/**
	 * Fill in the dynamic elements of the header nav.
	 */
	postCreate: function() {
		this._addCurrentProjectArea();
		if (!this._projectAreas) {
			this._addLoading();
			if (Util.isAdminUI()) {
				this._onclickHandle = this.connect(this.projectAreaSelector, "onclick", this._getOtherProjectAreas);
				this._timeoutHandle = setTimeout(dojo.hitch(this, this._getOtherProjectAreas), 6000);
			} else {
				this._getOtherProjectAreas();
			}
		} else if (this._projectAreas.length > 0) {
			var selectIndex = 1;
			for (var i = 0; i < this._projectAreas.length; i++) {
				if (!this._projectAreas[i].archived && this._projectAreas[i].myProjectArea && this.label != this._projectAreas[i].name) {
					this._addProjectArea(this._projectAreas[i],selectIndex);
					selectIndex++;
				}
			}
		} else {
			
		}
		this._addAdminOption();
		
		this.connect(this.projectAreaSelector, "onchange", this._selectProjectArea);	
		this.selectedIndex = this.projectAreaSelector.selectedIndex;
	},
	_getOtherProjectAreas: function() {
		if (this._projectAreasFetched) return;
		
		clearTimeout(this._timeoutHandle);
		this.disconnect(this._onclickHandle);
		this._timeoutHandle = null;
		this._onclickHandle = null;
		
		var srh = new ServiceResponseHandler(this, "_populateProjectAreas", "_noProjectAreas");
		var authContribId = getAuthenticatedUserId();
		var serviceRequest = new ServiceRequest(SERVICE_URI, "getAllProjectAreas", {hideArchivedProjects: true, userId: authContribId});
		TeamServerClient.invokeService(serviceRequest, srh);
		
		this._projectAreasFetched = true;
	},
	_populateProjectAreas: function(projectAreas) {
		if(projectAreas == null || projectAreas.length === 0){
			this._noProjectAreas();
			return;
		}
		var areasToAdd = new Array();
		for (var i = 0; i < projectAreas.length; i++) {
			if (!projectAreas[i].archived && projectAreas[i].myProjectArea && this.label != projectAreas[i].name) {
				areasToAdd.push(projectAreas[i]);
			}
		}
		if (areasToAdd.length == 0) {
			this._noProjectAreas(true);
			return;	
		}
		var options = this.projectAreaSelector.options;

		if (Util.isAdminUI() || Util.userIsAdmin()) {
			options[options.length-1] = null;
			options[options.length-1] = null;
		} 
		if (options.length > 0) {
			options[options.length-1] = null;
		}

		var selectIndex = 0;
		if (Util.isProjectAreaApplication()) {
			var option = new Option(this.messages.selectProject, this.messages.selectProject, true, true);
			option.className = "separator";
			dojo.addClass(this.projectAreaSelector,"com-ibm-team-repository-projectSelector-grayed");
			options[0] = option;
			this._addSeparator();
			selectIndex = 2;
		}
		
		for (var i = 0; i < areasToAdd.length; i++) {
			this._addProjectArea(areasToAdd[i],selectIndex);
			selectIndex++;
		}
		this._addAdminOption();

		this.selectedIndex = this.projectAreaSelector.selectedIndex;
	},
	_addProjectArea: function(projectArea, i) {
		if (projectArea.name.toLowerCase() == this.label.toLowerCase() || "the " + projectArea.name.toLowerCase() == this.label.toLowerCase()) {
			this.projectAreaSelector.options[i] = new Option(projectArea.name,projectArea.name,true,true);
			this.projectAreaSelector.options[i].className = "selected";
			this.projectAreaSelector.selectedIndex = i;
		} else 
			this.projectAreaSelector.options[i] = new Option(projectArea.name,projectArea.name);
	},
	_selectProjectArea: function(event) {
		var curUrl = new dojo._Url(location.href);
		
		if (curUrl.query) {  //check for debug mode
			var query = "?" + curUrl.query;	
		} else {
			var query = "";
		} 
		
		if (!Util.isAdminUI()) {
			var curPageId = PlatformUI.getWorkbench().currentPageId;
			if (curPageId) {
				var action = "#action=jazz.viewPage&id=" + curPageId;
			} else {
				var action = "";
			}
		} else {
			var action = "";
		}

		var projectAreaName = this.projectAreaSelector.options[this.projectAreaSelector.selectedIndex].value;
		if (projectAreaName == "disabled") {
			this.projectAreaSelector.options[this.selectedIndex].selected = "selected";
			return;
		}
		dojo.removeClass(this.projectAreaSelector,"com-ibm-team-repository-projectSelector-grayed");
		var href = "";
		if (projectAreaName == this.messages.projectAreas) {
			if (!Util.isProjectAreaApplication()) href = "#action=jazz.viewPage&id=com.ibm.team.process.projectAreaList";
			else href = net.jazz.ajax._contextRoot + "/web";
		} else if (projectAreaName == this.messages.admin) {
			href = net.jazz.ajax._contextRoot + "/admin" + query;
		} else {
			var encodedName = encodeURI(projectAreaName);
			encodedName = encodedName.replace(/\?/g, "%3F"); // encodeURI doesn't encode "?".  We need to force the encoding here.
			href = net.jazz.ajax._contextRoot + "/web/projects/" + encodedName + query + action;
		}
		
		if (PlatformUI.getWorkbench().shouldVetoNavigation(href)) {
			this._selectEntry(this.selectedIndex);
		} else {
			if (projectAreaName == this.messages.projectAreas && !Util.isProjectAreaApplication()) {
				this._selectEntry(this.selectedIndex); //re-highlight current project area
			} else {
				// highlight selected entry and store index
				this.projectAreaSelector.options[this.selectedIndex].className = "";
				this.selectedIndex = this.projectAreaSelector.selectedIndex;
				this.projectAreaSelector.options[this.selectedIndex].className = "selected";
			}
			location.href = href;
		}	
	},
	_selectEntry: function(index) {
		this.projectAreaSelector.options[this.projectAreaSelector.selectedIndex].className = "";
		this.projectAreaSelector.options[index].className = "selected";
		this.projectAreaSelector.options[index].selected = "selected";
	},
	_addLoading: function() {
		var loading = new Option(this.messages.loading, "disabled");
		loading.className = "separator";
		this.projectAreaSelector.options[this.projectAreaSelector.options.length] = loading;
	},
	_addProjectAreaOption: function() {
		var option = new Option(this.messages.projectAreas, this.messages.projectAreas); 
		if (Util.isProjectAreaApplication()) {
			option.className = "selected";	
		}
		this.projectAreaSelector.options[this.projectAreaSelector.options.length] = option;
	},
	_addSeparator: function() {
		var separator = new Option("-------------------------------","disabled");
		separator.disabled = "disabled";
		separator.className = "separator";
		this.projectAreaSelector.options[this.projectAreaSelector.options.length] = separator;
	},
	_addCurrentProjectArea: function() {
		if (!Util.isAdminUI() && !Util.isProjectAreaApplication()) {
			if (this.archived) {
				var option = new Option(dojo.string.substitute(this.messages.archivedProject, [ this.label ]), dojo.string.substitute(this.messages.archivedProject, [ this.label ]), true, true);	
			} else {
				var option = new Option(this.label, this.label, true, true);
			}
			option.className = "selected"
			this.projectAreaSelector.options[this.projectAreaSelector.options.length] = option;
			this.projectAreaSelector.selectedIndex = this.projectAreaSelector.options.length - 1;
		}
	},
	_addAdminOption: function() {
		if (Util.isAdminUI() || Util.userIsAdmin()) {
			this._addSeparator();
		
			if (Util.isAdminUI()) {
				var option = new Option(this.messages.admin, this.messages.admin, true, true);
				option.className = "selected";
				this.projectAreaSelector.options[this.projectAreaSelector.options.length] = option;
				this.projectAreaSelector.selectedIndex = this.projectAreaSelector.options.length - 1;
			} else 
				this.projectAreaSelector.options[this.projectAreaSelector.options.length] = new Option(this.messages.admin, this.messages.admin);
		}
	},
	_noProjectAreas: function(noPersonal) {
		this.projectAreaSelector.options[0].innerHTML = this.messages.noProjects;
		this.projectAreaSelector.options[0].text = this.messages.noProjects;
		if (this.projectAreaSelector.selectedIndex == 0) {
			dojo.addClass(this.projectAreaSelector,"com-ibm-team-repository-projectSelector-grayed");
		}
	}
});
})();
