/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005-2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.repository.web.ui.internal.wizard.EnableEmail");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.ServerPropertyService");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.ConfigUtility");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.ServerPropertyServiceRow");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("dojo.i18n");

dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.wizard", "EnableEmailMessages");


(function() {

var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var ServerPropertyService = com.ibm.team.repository.web.ui.internal.admin.server.ServerPropertyService;
var ConfigUtility = com.ibm.team.repository.web.ui.internal.admin.server.ConfigUtility;
var ServerPropertyServiceRow = com.ibm.team.repository.web.ui.internal.admin.server.ServerPropertyServiceRow;

var EMAIL_SERVICE_NAME = "com.ibm.team.repository.service.internal.MailerService";
var EMAIL_ENABLED_NAME = "notification.mail.enabled";
var RESTRICT_EMAIL_NAME = "notification.mail.restrict_to_whitelist";
var WHITELIST_NAME = "notification.mail.whitelist";

var DISPLAY_PROPS = [];
DISPLAY_PROPS.EMAIL = [
	"notification.mail.smtp.server",
	"notification.mail.smtp.server.port",
	"notification.mail.smtp.user",
	"notification.mail.smtp.password",
	"notification.mail.from",
	"notification.mail.from_name",
	"notification.mail.reply", 
	"notification.mail.smtp.starttls" 
];
DISPLAY_PROPS.WHITE_LIST = [
	"notification.mail.whitelist"
];

var messages;

/*          	TEST/REVERT/NEXT/EDIT
LOADING        		N   N   N   N
TESTING        		N   N   N   N
DISABLED       		N   Y   Y   Y
PENDING_OK 	      	Y   ?   N   Y 
PENDING_ERR 	    N   Y   N   Y
PENDING_WARN  	    Y   Y   N   Y
CONNECTED_OK        N   Y   Y   Y
CONNECTED_ERR       N   Y   N   Y
CONNECTED_WARN      N   Y   Y   Y  
*/
var STATUS = {
	LOADING: {test: false, revert: false, next: false, edit: false, code: "PROGRESS",
		message: 'loading'},  
	TESTING: {test: false, revert: false, next: false, edit: false, code: "PROGRESS", 
		message: 'testing'},  
	DISABLED: {test: false, revert: true, next: true, edit: true, code: "OK", 
		message: 'disabled'},
	PENDING_OK: {test: true, revert: true, next: false, edit: true, code: "INFO", 
		message: 'pendingOk'},  
	PENDING_ERR: {test: false, revert: true, next: false, edit: true, code: "ERROR",
		message: 'pendingErr'}, 
	PENDING_WARN: {test: true, revert: true, next: false, edit: true, code: "WARNING",
		message: 'pendingWarn'}, 
	CONNECTED_OK: {test: true, revert: true, next: true, edit: true, code: "OK",  
		message: 'connectedOk'},
	CONNECTED_ERR: {test: true, revert: true, next: false, edit: true, code: "ERROR",
		message: 'connectedErr'}, 
	CONNECTED_WARN: {test: true, revert: true, next: true, edit: true, code: "WARNING",
		message: 'connectedWarn'}, 
	SAVE_ERR: {test: true, revert: true, next: false, edit: true, code: "ERROR",
		message: 'saveErr'}  
};

dojo.declare("com.ibm.team.repository.web.ui.internal.wizard.EnableEmail",  [ConfigUtility, dijit._Widget, dijit._Templated], {

	templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/wizard/templates/EnableEmail.html"),
	
	constructor: function() {
		this.messages = messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.wizard", "EnableEmailMessages");
	},
	
	postCreate: function() {
		com.ibm.team.repository.web.ui.internal.wizard.EnableEmail.superclass.postCreate.apply(this);
		this._navigation = this.wizardNav;
		this._navigation.enablePrevious(true);
		dojo.connect(this._navigation, "requestNextApproval", this, function() {
			this._navigation.enableNext(false);	
			var invokedSave = this._saveHandler();
			if (!invokedSave) {
				this._navigation.processNext();
				this._navigation.enableNext(true);
			}
		});
		// NOTE: no changes require server restart 
		dojo.connect(this, "_submitConfigCallback", this, function() {
			this._initialLoadedState = STATUS.CONNECTED_OK;
			this._setState(STATUS.CONNECTED_OK);
			this._navigation.processNext();
			this._navigation.enableNext(true);
		}); 
		dojo.connect(this, "_submitConfigError", this, function(){
			this._setState(STATUS.SAVE_ERR);
		}); 
		this._setState(STATUS.LOADING);
		this._initializeContent();
	},
	
	// INITAL LOAD METHODS
	_initializeContent: function () {
		var serviceConfigProps = {
			serviceInfo: {
				implementationClassName: EMAIL_SERVICE_NAME
			},
			enableEdits: true,
			displayIn: this._hiddenTableBody,
			descriptionMode: "table", 
			initialMode: "edit"
		}
		this._emailConfig = new ServerPropertyService(serviceConfigProps);
		dojo.connect(this._emailConfig, "_markChanged", this, function() {
			this._setState(STATUS.PENDING_OK);
		});
		dojo.connect(this._emailConfig, "_checkUnchanged", this, function () {
				if(this._emailConfig._changed) {
					this._setState(STATUS.PENDING_OK);
				} else {
					this._setState(this._initialLoadedState);
				}
			});
		dojo.connect(this._emailConfig, "_markInvalid", this, "_handleInvalidInput");
		dojo.connect(this._emailConfig, "_show", this, "_initializePropertiesAfterLoaded");
	},
	
	_initializePropertiesAfterLoaded: function () {
		this._initializePropertyDisplay(EMAIL_ENABLED_NAME, this._componentEmailTableBody, DISPLAY_PROPS.EMAIL);
		this._initializePropertyDisplay(RESTRICT_EMAIL_NAME, this._componentWhiteListTableBody, DISPLAY_PROPS.WHITE_LIST);
		this._replaceWhitelistDescription();
		
		this._synchronizeSelectValue();
		this._displayRelevantSections();
		this._testConnection();
	},	
	
	_replaceWhitelistDescription: function () {
		var whitelistProperty = this._emailConfig.getServerPropertyElement("notification.mail.whitelist");
		dojox.data.dom.removeChildren(whitelistProperty._propertyDescription);
		whitelistProperty._propertyDescription.appendChild(document.createTextNode(messages.whitelist));
	},
	
	_initializePropertyDisplay: function (propertyName, attachPoint, displayProps) {
		var targetProperty = this._emailConfig.getServerPropertyElement(propertyName);
		// does it need the next line?
		targetProperty._editValueNode.value = targetProperty._currentValue;
		dojo.connect(targetProperty, "_markInvalid", this, function (msg) {
			this._handleInvalidInput(msg);
			this._showConnectionErrors([msg]);
		});
		var targetPropertyServiceRow = new ServerPropertyServiceRow({
			_enableToggle: false,
			rowClass: ["indentNone editProperties"],
			_descriptionMode: "table"
		});
		attachPoint.appendChild(targetPropertyServiceRow.domNode);
		
		var longExists = false;
		var shortExists = false;
		for (var i=0; i < displayProps.length; i++) {
			var propertyElement = this._emailConfig.getServerPropertyElement(displayProps[i]);
			if (propertyElement.elementInfo.propertyType === "STRING") {
				longExists = true;
				var table = targetPropertyServiceRow._longPropertiesTableBody;
			} else {
				shortExists = true;
				var table = targetPropertyServiceRow._shortPropertiesTableBody;
			}
			table.appendChild(propertyElement.domNode);
		};
		if (!longExists) {
			targetPropertyServiceRow._longPropertiesTableHeader.style.display = "none";
		}
		if (!shortExists) {
			targetPropertyServiceRow._shortPropertiesTableHeader.style.display = "none";
		}
	},
		
	// HANDLE USER INTERACTION METHODS
	 onInnerButtonAreaClick: function(e) {
		var button = e.target;
		if (button === this._revertButton) {
			this._revert();
			this._displayRelevantSections();
		} else if (button === this._testConnectionButton){
			this._testConnection();
		} 
	}, 
	
	_handleEmailEnabledChange: function (event) {
		this._handleSelectChange(event, EMAIL_ENABLED_NAME);
	},
	
	_handleWhitelistChange: function(event){
		var newSelectValue = event.target.value;
		
		if (newSelectValue === "false" && this._emailConfig.getEditedValueForProperty(WHITELIST_NAME).length > 0) {
			var whitelistProperty = this._emailConfig.getServerPropertyElement(WHITELIST_NAME);
			whitelistProperty._editValueNode.value = "";
			if (whitelistProperty._currentValue.length > 0) {
				whitelistProperty._markChanged();
			}
		}
		this._handleSelectChange(event, RESTRICT_EMAIL_NAME);
	}, 
	
	_handleSelectChange: function (event, propertyName) {
		var newSelectValue = event.target.value;
		var targetProperty = this._emailConfig.getServerPropertyElement(propertyName);
		targetProperty._editValueNode.value = newSelectValue;
		
		if (targetProperty._savedValue != newSelectValue) {
			targetProperty._markChanged();
		} else {
			targetProperty._markUnchanged();
		}
		targetProperty._checkValidity();
		this._displayRelevantSections();
	},
	
	_handleInvalidInput: function (msg){
		if (msg.code === "ERROR") {
			this._setState(STATUS.PENDING_ERR);
		} else {
			this._setState(STATUS.PENDING_WARN);
		}
	}, 
	
	// TEST CONNECTION METHODS
	_testConnection: function() {
		this._setState(STATUS.TESTING);
		var srh = new ServiceResponseHandler(this, "_testConnectionCallback", "_testConnectionError");
		this._emailConfig.testRobustValidity(srh);
	},
	
	_testConnectionCallback: function(msg) {
		var connected = null;
		if (msg != null) {
			connected = STATUS.CONNECTED_WARN;
			for (var i = 0; i < msg.length; i++) {
				if (msg[i].code === "ERROR") {
					connected = STATUS.CONNECTED_ERR;
				}
			}
		} else {
			connected = STATUS.CONNECTED_OK;
		}
		if (this._initialLoadedState == null) {
			this._initialLoadedState = connected;
		}
		this._setState(connected);
		if (msg != null) {
			this._showConnectionErrors(msg);
		}
	},
	
	_testConnectionError: function(errorObj) {
		this._setState(STATUS.CONNECTED_ERR);
		this._showConnectionErrors(messages.connectionError + errorObj.message);
	},
	
	_revert: function() {
		this._emailConfig.restoreSaved();
		this._synchronizeSelectValue();
		this._setState(this._initialLoadedState);
	},

	// HANDLE STATE METHODS
	_setState: function(newState) {
		var state = newState;
		// check required for initial load before service properties are loaded
		if (this._emailConfig != null) {
			if(this._emailConfig.getEditedValueForProperty(EMAIL_ENABLED_NAME) != "true") {
				state = STATUS.DISABLED;
			}
			this._allowPropertyEdit(state.edit); 
		}
		var enableRevert = state.revert;
		if (enableRevert && !this._emailConfig._changed) {
			enableRevert = false;
		}
		this._revertButton.disabled = (!enableRevert);
		this._testConnectionButton.disabled = (!state.test);
		this._navigation.enableNext(state.next);
		this._showConnectionStatus(state.code, messages[state.message]);
	},
				
	// UPDATE VIEW METHODS	
	_displayRelevantSections: function() {
		if (this._emailConfig.getEditedValueForProperty(EMAIL_ENABLED_NAME) === "true"){
			this._smtpPropertiesSection.style.display = "block";
			this._whitelistInnerStep.style.display = "block";
			if (this._emailConfig.getEditedValueForProperty(RESTRICT_EMAIL_NAME) === "true") {
				this._whitelistPropertiesSection.style.display = "block";
			} else {
				this._whitelistPropertiesSection.style.display = "none";
			}
		} else {
			this._whitelistInnerStep.style.display = "none";
			this._smtpPropertiesSection.style.display = "none";
		}
	}, 
	
	_allowPropertyEdit: function (allowEdits){
		var props = this._emailConfig._properties;
		for (var i=0; i < props.length; i++) {
			props[i]._editValueNode.disabled = !allowEdits;
		};	 
		this._selectEmailEnabled.disabled = !allowEdits;
		this._selectWhitelist.disabled = !allowEdits;
	},
	
	_synchronizeSelectValue: function() {
		var enabledProp = this._emailConfig.getServerPropertyElement(EMAIL_ENABLED_NAME);
		this._selectEmailEnabled.value = enabledProp._editValueNode.value;
		var restrictProp = this._emailConfig.getServerPropertyElement(RESTRICT_EMAIL_NAME);
		this._selectWhitelist.value = restrictProp._editValueNode.value;
	},
	
	_showConnectionStatus: function(code, statusMsg) {
		dojox.data.dom.removeChildren(this._stepStatus);
		
		var status = document.createElement("div");
		var indication = document.createElement("div");
		status.className = "stepStatus " + code;
		indication.className = "stepStatus statusMsg " + code;
		
		var statusImage = document.createElement("div");
		statusImage.className = "statusImage " + code+ "img";
		status.appendChild(statusImage);
		
		var message = document.createElement("div");
		dojo.addClass(message,"statusMsg");
		message.innerHTML = statusMsg;
		indication.innerHTML = statusMsg;
		status.appendChild(message);
		
		this._stepStatus.appendChild(status);
		this._setStatusIndication(code, indication);
	}, 
	
	_setStatusIndication: function (code, node) {
		if (code != "INFO") {
			dojox.data.dom.removeChildren(this.statusIndicationArea);
			dojo.toggleClass(this.statusIndicationArea, "noDisplay", false);
			this.statusIndicationArea.appendChild(node);
			if (code != "PROGRESS") {
				setTimeout(dojo.hitch(this, function(){
					dojo.toggleClass(this.statusIndicationArea, "noDisplay", true);
				}), 3500);
			}
		}
	},
	
	_showConnectionErrors: function(msg) {
		for (var i = 0; i < msg.length; i++) {
			var message = document.createElement("div");
			message.innerHTML = msg[i].message;			
			message.className = "stepStatus validationDetails " + msg[i].code;
			this._stepStatus.appendChild(message);
		}
	}, 
		
	// SUPER ConfigUtility METHODS	
	_getChanges: function () {
		var changes = new Array();
		var properties = new Array();
		if (this._emailConfig._changed) {
			var changeObj = this._emailConfig._getChanges();
			changes = changeObj["changes"];
			properties = changeObj["properties"];
		}
		return {
			"changes": changes,
			"properties": properties
		};
	}
});

})();